@@ -7,13 +7,13 @@ import asyncio
 import base64
 import logging
 import os
+import resource
 import shutil
 import struct
 import tempfile
 from dataclasses import dataclass
 from os.path import join as path_join
 from typing import Dict, Any, Union, List, Set
-
 import msgpack
 import psutil
 from inginious.agent.docker_agent._docker_interface import DockerInterface
@@ -23,7 +23,7 @@ from inginious.agent.docker_agent._docker_runtime import DockerRuntime
 from inginious.agent.docker_agent._timeout_watcher import TimeoutWatcher
 from inginious.common.asyncio_utils import AsyncIteratorWrapper, AsyncProxy
 from inginious.common.base import id_checker, id_checker_tests
-from inginious.common.filesystems.provider import FileSystemProvider
+from inginious.common.filesystems import FileSystemProvider
 from inginious.common.messages import BackendNewJob, BackendKillJob
 
 
@@ -49,6 +49,8 @@ class DockerRunningJob:
     run_cmd: str
     assigned_external_ports: List[int]
     student_containers: Set[str]  # container ids of student containers
+    enable_network: bool
+    ssh_allowed: bool
 
 
 @dataclass
@@ -57,10 +59,14 @@ class DockerRunningStudentContainer:
     parent_info: DockerRunningJob
     socket_id: str
     write_stream: Any
+    ssh: bool
+    ports: Dict[int, int]  # internal port -> external port mapping
+    assigned_external_ports: List[int]
 
 
 class DockerAgent(Agent):
-    def __init__(self, context, backend_addr, friendly_name, concurrency, tasks_fs: FileSystemProvider, address_host=None, external_ports=None, tmp_dir="./agent_tmp", runtimes=None):
+    def __init__(self, context, backend_addr, friendly_name, concurrency, tasks_fs: FileSystemProvider,
+                 address_host=None, external_ports=None, tmp_dir="./agent_tmp", runtimes=None, ssh_allowed=False):
         """
         :param context: ZeroMQ context for this process
         :param backend_addr: address of the backend (for example, "tcp://127.0.0.1:2222")
@@ -72,13 +78,17 @@ class DockerAgent(Agent):
         :param tmp_dir: temp dir that is used by the agent to start new containers
         :param type: type of the container ("docker" or "kata")
         :param runtime: runtime used by docker (the defaults are "runc" with docker or "kata-runtime" with kata)
+        :param ssh_allowed: boolean to make this agent accept tasks with ssh or not
         """
-        super(DockerAgent, self).__init__(context, backend_addr, friendly_name, concurrency, tasks_fs)
+        super(DockerAgent, self).__init__(context, backend_addr, friendly_name, concurrency, tasks_fs,
+                                          ssh_allowed=ssh_allowed)
 
         self._runtimes = {x.envtype: x for x in runtimes} if runtimes is not None else None
 
         self._logger = logging.getLogger("inginious.agent.docker")
 
+        self._concurrency = concurrency
+
         self._max_memory_per_slot = int(psutil.virtual_memory().total / concurrency / 1024 / 1024)
 
         # Temp dir
@@ -125,7 +135,8 @@ class DockerAgent(Agent):
 
         if self._address_host is None and len(self._containers) != 0:
             self._logger.info("Guessing external host IP")
-            available_bare_container_images = [image for envtype_containers in self._containers.values() for image in envtype_containers.values()]
+            available_bare_container_images = [image for envtype_containers in self._containers.values() for image in
+                                               envtype_containers.values()]
             if len(available_bare_container_images) != 0:
                 self._address_host = await self._docker.get_host_ip(available_bare_container_images[0]["id"])
             else:
@@ -213,7 +224,8 @@ class DockerAgent(Agent):
         since = None  # last time we saw something. Useful if a restart happens...
         while not shutdown:
             try:
-                source = AsyncIteratorWrapper(self._docker.sync.event_stream(filters={"event": ["die", "oom"]}, since=since))
+                source = AsyncIteratorWrapper(
+                    self._docker.sync.event_stream(filters={"event": ["die", "oom"]}, since=since))
                 self._logger.info("Docker event stream started")
                 async for i in source:
                     since = i.get('time', since)  # update time if available.
@@ -222,6 +234,7 @@ class DockerAgent(Agent):
                         container_id = i["id"]
                         try:
                             retval = int(i["Actor"]["Attributes"]["exitCode"])
+
                         except asyncio.CancelledError:
                             raise
                         except:
@@ -245,12 +258,17 @@ class DockerAgent(Agent):
                                 pass
                     else:
                         raise TypeError(str(i))
-                raise Exception("Docker stopped feeding the event stream. This should not happen. Restarting the event stream...")
+                raise Exception(
+                    "Docker stopped feeding the event stream. This should not happen. Restarting the event stream...")
             except asyncio.CancelledError:
                 shutdown = True
             except:
                 self._logger.exception("Exception in _watch_docker_events")
 
+    def __get_fd_limit(self):
+        """Get soft and hard fd limits per simultaneous jobs"""
+        fd_limits = resource.getrlimit(resource.RLIMIT_NOFILE)
+        return int(fd_limits[0] / self._concurrency), int(fd_limits[1] / self._concurrency)
 
     def __new_job_sync(self, message: BackendNewJob, future_results):
         """ Synchronous part of _new_job. Creates needed directories, copy files, and starts the container. """
@@ -263,6 +281,7 @@ class DockerAgent(Agent):
 
         try:
             enable_network = message.environment_parameters.get("network_grading", False)
+            ssh_allowed = message.environment_parameters.get("ssh_allowed", False)
             limits = message.environment_parameters.get("limits", {})
             time_limit = int(limits.get("time", 30))
             hard_time_limit = int(limits.get("hard_time", None) or time_limit * 3)
@@ -276,8 +295,9 @@ class DockerAgent(Agent):
 
         if not course_fs.exists() or not task_fs.exists():
             self._logger.warning("Task %s/%s unavailable on this agent", course_id, task_id)
-            raise CannotCreateJobException('Task unavailable on agent. Please retry later, the agents should synchronize soon. '
-                             'If the error persists, please contact your course administrator.')
+            raise CannotCreateJobException(
+                'Task unavailable on agent. Please retry later, the agents should synchronize soon. '
+                'If the error persists, please contact your course administrator.')
 
         # Check for realistic memory limit value
         if mem_limit < 20:
@@ -285,7 +305,8 @@ class DockerAgent(Agent):
         elif mem_limit > self._max_memory_per_slot:
             self._logger.warning("Task %s/%s ask for too much memory (%dMB)! Available: %dMB", course_id, task_id,
                                  mem_limit, self._max_memory_per_slot)
-            raise CannotCreateJobException('Not enough memory on agent (available: %dMB). Please contact your course administrator.' % self._max_memory_per_slot)
+            raise CannotCreateJobException(
+                'Not enough memory on agent (available: %dMB). Please contact your course administrator.' % self._max_memory_per_slot)
 
         if environment_type not in self._containers or environment_name not in self._containers[environment_type]:
             self._logger.warning("Task %s/%s ask for an unknown environment %s/%s", course_id, task_id,
@@ -295,13 +316,14 @@ class DockerAgent(Agent):
         environment = self._containers[environment_type][environment_name]["id"]
         runtime = self._containers[environment_type][environment_name]["runtime"]
 
-        ports_needed = list(self._containers[environment_type][environment_name]["ports"])  # copy, as we modify it later!
+        ports_needed = list(
+            self._containers[environment_type][environment_name]["ports"])  # copy, as we modify it later!
 
         if debug == "ssh" and 22 not in ports_needed:
             ports_needed.append(22)
 
         ports = {}
-        if len(ports_needed) > 0:
+        if len(ports_needed) > 0:  # if ssh_debug, put time limits to 30 min.
             time_limit = 30 * 60
             hard_time_limit = 30 * 60
         for p in ports_needed:
@@ -359,7 +381,9 @@ class DockerAgent(Agent):
         try:
             container_id = self._docker.sync.create_container(environment, enable_network, mem_limit, task_path,
                                                               sockets_path, course_common_path,
-                                                              course_common_student_path, runtime, ports)
+                                                              course_common_student_path,
+                                                              self.__get_fd_limit(), runtime,
+                                                              ports)
         except Exception as e:
             self._logger.warning("Cannot create container! %s", str(e), exc_info=True)
             shutil.rmtree(container_path)
@@ -388,7 +412,9 @@ class DockerAgent(Agent):
             course_common_student_path=course_common_student_path,
             run_cmd=run_cmd,
             assigned_external_ports=list(ports.values()),
-            student_containers=set()
+            student_containers=set(),
+            enable_network=enable_network,
+            ssh_allowed=ssh_allowed
         )
 
         self._containers_running[container_id] = info
@@ -417,7 +443,8 @@ class DockerAgent(Agent):
         await self._timeout_watcher.register_container(out.container_id, out.time_limit, out.hard_time_limit)
 
     async def create_student_container(self, parent_info, socket_id, environment_name,
-                                       memory_limit, time_limit, hard_time_limit, share_network, write_stream):
+                                       memory_limit, time_limit, hard_time_limit, share_network, write_stream, ssh,
+                                       run_as_root):
         """
         Creates a new student container.
         :param write_stream: stream on which to write the return value of the container (with a correctly formatted msgpack message)
@@ -435,18 +462,43 @@ class DockerAgent(Agent):
                 return
 
             environment = self._containers[environment_type][environment_name]["id"]
-            runtime = self._containers[environment_type][environment_name]["runtime"]
+
+            if run_as_root:
+                runtime_name = {k for k in self._runtimes if self._runtimes[k].run_as_root}.pop()
+                runtime = self._runtimes[runtime_name].runtime
+            else:
+                runtime = self._containers[environment_type][environment_name]["runtime"]
+
+            ports_needed = [22] if ssh else []
+            ports = {}
+            for p in ports_needed:
+                if not self._external_ports:
+                    self._logger.warning("User asked for a port but no one are available")
+                    if self._external_ports is not None:
+                        for port_to_free in ports:
+                            self._external_ports.add(ports[port_to_free])
+                    self._logger.exception(
+                        "Cannot create student container! No ports are available right now. Please retry later.")
+                    await self._write_to_container_stdin(write_stream, {"type": "run_student_retval", "retval": 254,
+                                                                        "socket_id": socket_id})
+                    raise CannotCreateJobException('No ports are available right now. Please retry later.')
+                ports[p] = self._external_ports.pop()
 
             try:
                 socket_path = path_join(parent_info.sockets_path, str(socket_id) + ".sock")
                 container_id = await self._docker.create_container_student(runtime, environment,
-                                                                           memory_limit, parent_info.student_path, socket_path,
+                                                                           memory_limit, parent_info.student_path,
+                                                                           socket_path,
                                                                            parent_info.systemfiles_path,
                                                                            parent_info.course_common_student_path,
-                                                                           parent_info.container_id if share_network else None)
+                                                                           parent_info.environment_type,
+                                                                           self.__get_fd_limit(),
+                                                                           parent_info.container_id if share_network else None,
+                                                                           ports)
             except Exception as e:
                 self._logger.exception("Cannot create student container!")
-                await self._write_to_container_stdin(write_stream, {"type": "run_student_retval", "retval": 254, "socket_id": socket_id})
+                await self._write_to_container_stdin(write_stream, {"type": "run_student_retval", "retval": 254,
+                                                                    "socket_id": socket_id})
 
                 if isinstance(e, asyncio.CancelledError):
                     raise
@@ -457,20 +509,25 @@ class DockerAgent(Agent):
                 container_id=container_id,
                 parent_info=parent_info,
                 socket_id=socket_id,
-                write_stream=write_stream
+                write_stream=write_stream,
+                ssh=ssh,
+                ports=ports,
+                assigned_external_ports=list(ports.values())
             )
 
             parent_info.student_containers.add(container_id)
             self._student_containers_running[container_id] = info
 
             # send to the container that the sibling has started
-            await self._write_to_container_stdin(write_stream, {"type": "run_student_started", "socket_id": socket_id})
+            await self._write_to_container_stdin(write_stream, {"type": "run_student_started", "socket_id": socket_id,
+                                                                "container_id": container_id})
 
             try:
                 await self._docker.start_container(container_id)
             except Exception as e:
                 self._logger.exception("Cannot start student container!")
-                await self._write_to_container_stdin(write_stream, {"type": "run_student_retval", "retval": 254, "socket_id": socket_id})
+                await self._write_to_container_stdin(write_stream, {"type": "run_student_retval", "retval": 254,
+                                                                    "socket_id": socket_id})
 
                 if isinstance(e, asyncio.CancelledError):
                     raise
@@ -484,6 +541,63 @@ class DockerAgent(Agent):
         except:
             self._logger.exception("Exception in create_student_container")
 
+    async def read_stream(self, reader_stream, buffer):
+        """Helper to read a read and put data on a buffer"""
+        msg_header = await reader_stream.readexactly(8)
+        outtype, length = struct.unpack_from('>BxxxL',
+                                             msg_header)  # format imposed by docker in the attach endpoint
+        if length != 0:
+            content = await reader_stream.readexactly(length)
+            if outtype == 1:  # stdout
+                buffer += content
+
+            if outtype == 2:  # stderr
+                self._logger.debug("Received stderr from containers:\n%s", content)
+            # 4 first bytes are the length of the message. If we have a complete message...
+            return buffer
+        else:
+            raise Exception("Wrong format message received")
+
+    def read_buffer(self, buffer, decode=True):
+        """Helper to read a buffer containing data from a stream"""
+        length = struct.unpack('!I', buffer[0:4])[0]
+        msg = buffer[4:4 + length]  # ... get it
+        buffer = buffer[4 + length:]  # ... withdraw it from the buffer
+        if decode:
+            msg = msgpack.unpackb(msg, use_list=False)
+        return buffer, msg
+
+    def buffer_has_data(self, buffer):
+        """Helper to know if a buffer still has available data"""
+        return len(buffer) > 4 and len(buffer) >= 4 + struct.unpack('!I', buffer[0:4])[0]
+
+    async def start_ssh(self, reader_stream, info):
+        """ Wait for ssh information from student_container and send ssh info to frontend """
+        buffer = bytearray()
+        try:
+            while not reader_stream.at_eof():  # CHECK HERE
+                buffer = await self.read_stream(reader_stream, buffer)
+                while self.buffer_has_data(buffer):
+                    buffer, msg = self.read_buffer(buffer)
+                    self._logger.debug("Received msg %s from container %s", msg["type"], info.container_id)
+                    if msg["type"] == "ssh_student":
+                        info_student = None
+                        if len(self._student_containers_running) > 0 and msg[
+                            "container_id"] in info.student_containers:
+                            info_student = self._student_containers_running[msg["container_id"]]
+                            await self.send_ssh_job_info(info.job_id, self._address_host, info_student.ports[22],
+                                                         msg["ssh_user"], msg["ssh_key"])
+                        else:
+                            self._logger.exception("Exception: no linked student_container running.")
+                            self._create_safe_task(self.handle_job_closing(info.container_id, -1,
+                                                                           manual_feedback="Trying to connect with ssh to a non-children student container !"))
+                        return
+        except asyncio.IncompleteReadError:
+            self._logger.debug("Container output ended with an IncompleteReadError; It was probably killed.")
+        except:
+            self._logger.exception("Received incorrect message from container %s (job id %s)", info.container_id,
+                                   info.job_id)
+
     async def _write_to_container_stdin(self, write_stream, message):
         """
         Send a message to the stdin of a container, with the right data
@@ -492,15 +606,38 @@ class DockerAgent(Agent):
         """
         msg = msgpack.dumps(message, use_bin_type=True)
         self._logger.debug("Sending %i bytes to container", len(msg))
-        write_stream.write(struct.pack('I', len(msg)))
+        write_stream.write(struct.pack('!I', len(msg)))
         write_stream.write(msg)
         await write_stream.drain()
 
+    async def _handle_student_container_outputs(self, student_reader_stream, grading_write_stream):
+        """ Receive outputs (stdout and stderr) from student_container and send them to grading_container without decoding """
+        buffer = bytearray()
+        try:
+            while not student_reader_stream.at_eof():
+                buffer = await self.read_stream(student_reader_stream, buffer)
+                while self.buffer_has_data(buffer):
+                    buffer, msg_encoded = self.read_buffer(buffer, decode=False)
+                    try:
+                        grading_write_stream.write(
+                            struct.pack('!I', len(msg_encoded)))  # Transfer the message without decoding it
+                        grading_write_stream.write(msg_encoded)
+                        await grading_write_stream.drain()
+                    except Exception as e:
+                        self._logger.info("Student container closed the stream")
+                        self._logger.info(e)
+                        return
+        except asyncio.IncompleteReadError:
+            self._logger.debug("Container output ended with an IncompleteReadError; It was probably killed.")
+            return
+        except:
+            self._logger.exception("Received incorrect message from student container")
+
     async def handle_running_container(self, info: DockerRunningJob, future_results):
         """ Talk with a container. Sends the initial input. Allows to start student containers """
         sock = await self._docker.attach_to_container(info.container_id)
         try:
-            read_stream, write_stream = await asyncio.open_connection(sock=sock.get_socket())
+            reader_stream, write_stream = await asyncio.open_connection(sock=sock.get_socket())
         except asyncio.CancelledError:
             raise
         except:
@@ -508,7 +645,8 @@ class DockerAgent(Agent):
             return None
 
         # Send hello msg
-        hello_msg = {"type": "start", "input": info.inputdata, "debug": info.debug, "envtypes": {x.envtype: x.shared_kernel for x in self._runtimes.values()}}
+        hello_msg = {"type": "start", "input": info.inputdata, "debug": info.debug,
+                     "envtypes": {x.envtype: x.shared_kernel for x in self._runtimes.values()}}
         if info.run_cmd is not None:
             hello_msg["run_cmd"] = info.run_cmd
         hello_msg["run_as_root"] = self._runtimes[info.environment_type].run_as_root
@@ -519,45 +657,92 @@ class DockerAgent(Agent):
 
         buffer = bytearray()
         try:
-            while not read_stream.at_eof():
-                msg_header = await read_stream.readexactly(8)
-                outtype, length = struct.unpack_from('>BxxxL', msg_header)  # format imposed by docker in the attach endpoint
-                if length != 0:
-                    content = await read_stream.readexactly(length)
-                    if outtype == 1:  # stdout
-                        buffer += content
+            student_containers_streams = {}
+            while not reader_stream.at_eof():
+                buffer = await self.read_stream(reader_stream, buffer)
+                while self.buffer_has_data(buffer):
+                    buffer, msg = self.read_buffer(buffer)
+                    try:
+                        self._logger.debug("Received msg %s from container %s", msg["type"], info.container_id)
+                        if msg["type"] == "run_student":
+                            # start a new student container
+                            environment = msg["environment"] or info.environment_name
+                            memory_limit = min(msg["memory_limit"] or info.mem_limit, info.mem_limit)
+                            time_limit = min(msg["time_limit"] or info.time_limit, info.time_limit)
+                            hard_time_limit = min(msg["hard_time_limit"] or info.hard_time_limit, info.hard_time_limit)
+                            share_network = msg["share_network"]
+                            socket_id = msg["socket_id"]
+                            ssh = msg["ssh"]
+                            run_as_root = msg["run_as_root"]
+                            assert "/" not in socket_id  # ensure task creator do not try to break the agent :-(
+                            if ssh and not (info.enable_network and info.ssh_allowed):
+                                self._logger.error(
+                                    "Exception: ssh for student requires to allow ssh and internet access in the task %s environment configuration tab",
+                                    info.job_id)
+                                self._create_safe_task(self.handle_job_closing(info.container_id, -1,
+                                                                               manual_feedback="ssh for student requires to allow ssh and internet access in the task environment configuration tab!"))
+                            else:
+                                self._create_safe_task(
+                                    self.create_student_container(info, socket_id, environment, memory_limit,
+                                                                  time_limit, hard_time_limit, share_network,
+                                                                  write_stream, ssh, run_as_root))
 
-                    if outtype == 2:  # stderr
-                        self._logger.debug("Received stderr from containers:\n%s", content)
+                        elif msg["type"] == "run_student_init":  # We use non docker-docker communication !
+                            if msg["student_container_id"] not in student_containers_streams:
+                                student_containers_streams[
+                                    msg["student_container_id"]] = await self.open_student_stream(
+                                    msg["student_container_id"])
+                            await self._write_to_container_stdin(
+                                student_containers_streams[msg["student_container_id"]][1],
+                                {"type": "run_student_init",
+                                 "socket_id": msg["socket_id"],
+                                 "command": msg["command"],
+                                 "teardown_script": msg["teardown_script"],
+                                 "student_container_id": msg[
+                                     "student_container_id"],
+                                 "working_dir": msg["working_dir"],
+                                 "ssh": msg["ssh"],
+                                 "user": msg["user"]})
 
-                    # 4 first bytes are the length of the message. If we have a complete message...
-                    while len(buffer) > 4 and len(buffer) >= 4+struct.unpack('I',buffer[0:4])[0]:
-                        msg_encoded = buffer[4:4 + struct.unpack('I', buffer[0:4])[0]]  # ... get it
-                        buffer = buffer[4 + struct.unpack('I', buffer[0:4])[0]:]  # ... withdraw it from the buffer
-                        try:
-                            msg = msgpack.unpackb(msg_encoded, use_list=False)
-                            self._logger.debug("Received msg %s from container %s", msg["type"], info.container_id)
-                            if msg["type"] == "run_student":
-                                # start a new student container
-                                environment = msg["environment"] or info.environment_name
-                                memory_limit = min(msg["memory_limit"] or info.mem_limit, info.mem_limit)
-                                time_limit = min(msg["time_limit"] or info.time_limit, info.time_limit)
-                                hard_time_limit = min(msg["hard_time_limit"] or info.hard_time_limit, info.hard_time_limit)
-                                share_network = msg["share_network"]
-                                socket_id = msg["socket_id"]
-                                assert "/" not in socket_id  # ensure task creator do not try to break the agent :-(
-                                self._create_safe_task(self.create_student_container(info, socket_id, environment, memory_limit,
-                                                                                     time_limit, hard_time_limit, share_network,
-                                                                                     write_stream))
-                            elif msg["type"] == "ssh_key":
-                                # send the data to the backend (and client)
-                                self._logger.info("%s %s", info.container_id, str(msg))
-                                await self.send_ssh_job_info(info.job_id, self._address_host, info.ports[22], msg["ssh_user"], msg["ssh_key"])
-                            elif msg["type"] == "result":
-                                # last message containing the results of the container
-                                result = msg["result"]
-                        except:
-                            self._logger.exception("Received incorrect message from container %s (job id %s)", info.container_id, info.job_id)
+                            if msg["ssh"]:
+                                await self.start_ssh(student_containers_streams[msg["student_container_id"]][0],
+                                                     info)  # If using ssh with kata: wait for ssh info and start ssh
+                            else:  # classical run_student (not ssh_student) with a kata runtime -> handle student_container outputs
+                                self._loop.create_task(self._handle_student_container_outputs(
+                                    student_containers_streams[msg["student_container_id"]][0], write_stream))
+
+                        elif msg["type"] in ["stdin", "student_signal"]:  # Simply transfer to student_container
+                            if msg["student_container_id"] not in student_containers_streams:
+                                student_containers_streams[
+                                    msg["student_container_id"]] = await self.open_student_stream(
+                                    msg["student_container_id"])
+                            await self._write_to_container_stdin(
+                                student_containers_streams[msg["student_container_id"]][1], msg)
+
+                        elif msg["type"] == "ssh_debug":
+                            # send the data to the frontend (and client) to reach grading_container
+                            self._logger.info("%s %s", info.container_id, str(msg))
+                            await self.send_ssh_job_info(info.job_id, self._address_host, info.ports[22],
+                                                         msg["ssh_user"], msg["ssh_key"])
+
+                        elif msg["type"] == "ssh_student":
+                            # send the data to the frontend (and client) to reach student_container
+                            info_student = None
+                            if len(self._student_containers_running) > 0 and msg[
+                                "container_id"] in info.student_containers:
+                                info_student = self._student_containers_running[msg["container_id"]]
+                            else:
+                                self._logger.exception("Exception: no linked student_container running.")
+                                self._create_safe_task(self.handle_job_closing(info.container_id, -1,
+                                                                               manual_feedback="Trying to connect with ssh to a non-children student container !"))
+                            self._logger.info("%s %s", info_student.container_id, str(msg))
+                            await self.send_ssh_job_info(info.job_id, self._address_host, info_student.ports[22],
+                                                         msg["ssh_user"], msg["ssh_key"])
+                        elif msg["type"] == "result":
+                            result = msg["result"]  # last message containing the results of the container
+                    except:
+                        self._logger.exception("Received incorrect message from container %s (job id %s)",
+                                               info.container_id, info.job_id)
         except asyncio.IncompleteReadError:
             self._logger.debug("Container output ended with an IncompleteReadError; It was probably killed.")
         except asyncio.CancelledError:
@@ -575,25 +760,34 @@ class DockerAgent(Agent):
         if not result:
             self._logger.warning("Container %s has not given any result", info.container_id)
 
+    async def open_student_stream(self, student_container_id):
+        student_sock = await self._docker.attach_to_container(student_container_id)
+        student_reader_stream, student_write_stream = await asyncio.open_connection(
+            sock=student_sock.get_socket())
+        stream = (student_reader_stream, student_write_stream)
+        return stream
+
     async def handle_student_job_closing(self, container_id, retval):
-        """
-        Handle a closing student container. Do some cleaning, verify memory limits, timeouts, ... and returns data to the associated grading
-        container
-        """
+        """ Handle a closing student container. Do some cleaning, verify memory limits, timeouts, ... and returns data to the associated grading container """
         try:
-            self._logger.debug("Closing student %s", container_id)
+            self._logger.debug("CLOSING STUDENT CONTAINER %s", container_id)
             try:
                 info = self._student_containers_running[container_id]
                 del self._student_containers_running[container_id]
             except asyncio.CancelledError:
                 raise
             except:
-                self._logger.warning("Student container %s that has finished(p1) was not launched by this agent", str(container_id), exc_info=True)
+                self._logger.warning("Student container %s that has finished(p1) was not launched by this agent",
+                                     str(container_id), exc_info=True)
                 return
 
             # Delete remaining student containers
             info.parent_info.student_containers.remove(container_id)
 
+            # Allow other container to reuse the external ports this container has finished to use
+            for p in info.assigned_external_ports:
+                self._external_ports.add(p)
+
             killed = await self._timeout_watcher.was_killed(container_id)
             if container_id in self._containers_killed:
                 killed = self._containers_killed[container_id]
@@ -605,7 +799,8 @@ class DockerAgent(Agent):
                 retval = 252
 
             try:
-                await self._write_to_container_stdin(info.write_stream, {"type": "run_student_retval", "retval": retval, "socket_id": info.socket_id})
+                await self._write_to_container_stdin(info.write_stream, {"type": "run_student_retval", "retval": retval,
+                                                                         "socket_id": info.socket_id})
             except asyncio.CancelledError:
                 raise
             except:
@@ -623,7 +818,7 @@ class DockerAgent(Agent):
         except:
             self._logger.exception("Exception in handle_student_job_closing")
 
-    async def handle_job_closing(self, container_id, retval):
+    async def handle_job_closing(self, container_id, retval, manual_feedback=None):
         """
         Handle a closing student container. Do some cleaning, verify memory limits, timeouts, ... and returns data to the backend
         """
@@ -635,9 +830,9 @@ class DockerAgent(Agent):
             except asyncio.CancelledError:
                 raise
             except:
-                self._logger.warning("Container %s that has finished(p1) was not launched by this agent", str(container_id), exc_info=True)
+                self._logger.warning("Container %s that has finished(p1) was not launched by this agent",
+                                     str(container_id), exc_info=True)
                 return
-
             # Close sub containers
             for student_container_id_loop in info.student_containers:
                 # little hack to ensure the value of student_container_id_loop is copied into the closure
@@ -649,6 +844,7 @@ class DockerAgent(Agent):
                         raise
                     except:
                         pass  # ignore
+
                 self._create_safe_task(close_and_delete(student_container_id_loop))
 
             # Allow other container to reuse the external ports this container has finished to use
@@ -670,7 +866,7 @@ class DockerAgent(Agent):
             custom = {}
             tests = {}
             archive = None
-            state = ""
+            state = info.inputdata.get("@state", "")  # init state to previous state
 
             if killed is not None:
                 result = killed
@@ -691,7 +887,7 @@ class DockerAgent(Agent):
                     for key, item in return_value.items():
                         if not isinstance(item, accepted_types[key]):
                             raise Exception("Feedback file is badly formatted.")
-                        elif accepted_types[key] == dict and key != "custom": #custom can contain anything:
+                        elif accepted_types[key] == dict and key != "custom":  # custom can contain anything:
                             for sub_key, sub_item in item.items():
                                 if not keys_fct[key](sub_key) or isinstance(sub_item, dict):
                                     raise Exception("Feedback file is badly formatted.")
@@ -739,7 +935,12 @@ class DockerAgent(Agent):
                 pass  # todo: run a docker container to force removal
 
             # Return!
-            await self.send_job_result(info.job_id, result, error_msg, grade, problems, tests, custom, state, archive, stdout, stderr)
+            if retval == -1 and manual_feedback is not None and isinstance(manual_feedback, str):
+                await self.send_job_result(info.job_id, result, manual_feedback, grade, problems, tests, custom, state,
+                                           archive, stdout, stderr)
+            else:
+                await self.send_job_result(info.job_id, result, error_msg, grade, problems, tests, custom, state,
+                                           archive, stdout, stderr)
 
             # Do not forget to remove data from internal state
             del self._container_for_job[info.job_id]
@@ -758,7 +959,7 @@ class DockerAgent(Agent):
                 self._logger.warning("Cannot kill container for job %s because it is not running", str(message.job_id))
                 # Ensure the backend/frontend receive the info that the job is done. This will be ignored in the worst
                 # case.
-                await self.send_job_result(message.job_id, "killed")
+                await self.send_job_result(message.job_id, "killed", state=message.state)
         except asyncio.CancelledError:
             raise
         except:
@@ -793,5 +994,7 @@ class DockerAgent(Agent):
                         self._logger.info("Using %s as runtime with parameters %s", runtime, str(v))
                         retval[v.envtype] = v
                     else:
-                        self._logger.warning("%s was detected as a runtime; it would duplicate another one, so we ignore it. %s", runtime, str(v))
+                        self._logger.warning(
+                            "%s was detected as a runtime; it would duplicate another one, so we ignore it. %s",
+                            runtime, str(v))
         return retval
