@@ -1,5 +1,9 @@
-# coding=utf-8
-import web
+# -*- coding: utf-8 -*-
+#
+# This file is part of INGInious. See the LICENSE and the COPYRIGHTS files for
+# more information about the licensing of this file.
+
+import flask
 from lti import ToolProvider
 
 
@@ -10,13 +14,13 @@ class LTIWebPyToolProvider(ToolProvider):
 
     @classmethod
     def from_webpy_request(cls, secret=None):
-        params = web.webapi.rawinput("POST")
-        headers = web.ctx.env.copy()
+        params = flask.request.form.copy()
+        headers = flask.request.headers.environ.copy()
 
         headers = dict([(k, headers[k])
                         for k in headers if
                         k.upper().startswith('HTTP_') or
                         k.upper().startswith('CONTENT_')])
 
-        url = web.ctx.home + web.ctx.fullpath
+        url = flask.request.url
         return cls.from_unpacked_request(secret, params, url, headers)
