@@ -168,7 +168,7 @@ class CourseStudentListPage(INGIniousAdminPage):
                         self.database.groups.replace_one({"_id": group["_id"]}, group)
                     self.database.courses.find_one_and_update({"_id": course.get_id()}, {"$set": {"students": []}})
                 else:
-                    self.user_manager.course_unregister_user(course, data["username"])
+                    self.user_manager.course_unregister_user(course.get_id(), data["username"])
             except:
                 pass
         elif "register_student" in data:
@@ -180,9 +180,9 @@ class CourseStudentListPage(INGIniousAdminPage):
     def post_audiences(self, course, data, active_tab, msg, error):
         try:
             if 'audience' in data:
-                self.database.audiences.insert({"courseid": course.get_id(), "students": [],
-                                                 "tutors": [],
-                                                 "description": data['audience']})
+                self.database.audiences.insert_one({"courseid": course.get_id(), "students": [],
+                                                    "tutors": [],
+                                                    "description": data['audience']})
                 msg["audiences"] = _("New audience created.")
                 active_tab = "tab_audiences"
 
