@@ -22,11 +22,11 @@ class CourseListPage(INGIniousPage):
     def show_page(self):
         """  Display main course list page """
         username = self.user_manager.session_username()
-        user_info = self.database.users.find_one({"username": username})
+        user_info = self.user_manager.get_user_info(username)
         all_courses = self.course_factory.get_all_courses()
 
         # Display
         open_courses = {courseid: course for courseid, course in all_courses.items() if course.is_open_to_non_staff()}
         open_courses = OrderedDict(sorted(iter(open_courses.items()), key=lambda x: x[1].get_name(self.user_manager.session_language())))
 
-        return self.template_helper.get_renderer().index(open_courses, user_info)
+        return self.template_helper.render("courselist.html", open_courses=open_courses, user_info=user_info)
